#include "CliArguments.h"
#include "Process.h"
#include "Wallpapers.h"
#include "makeProcess.h"
#include <exception>
#include <iostream>
#include <ostream>
#include <string>

int main(int argc, char** argv, char** envp) {

	try {
		CliArguments::CliArguments arguments{
			argc, argv,
			{
				{"path", {
					.description = "The path to the directory containing the wallpapers",
					.type = CliArguments::String,
					.order = 0,
					.direct = true
				}},
				{"output", {
					.aliases = {"o"},
					.description = "The outputs as shown with the command `swaymsg -t get_outputs`, by default the wallpaper will be applied to all outputs",
					.type = CliArguments::String,
				}},
				{"interval", {
					.aliases = {"i"},
					.description = "Specify the interval between wallpapers in seconds",
					.type = CliArguments::Int,
				}},
				{"never-random", {
					.description = "if set, wallpapers order is based on the names of the files",
					.type = CliArguments::Bool,
				}},
				{"lock", {
					.description = "Lock the screen using swaylock instead of applying a wallpaper",
					.type = CliArguments::Bool,
				}},
				{"help", {
					.aliases = {"h"},
					.description = "Show help",
					.type = CliArguments::Bool,
				}},
			}
		};

		if (!arguments.isValid()) {
			std::cout << arguments.error() << std::endl;
			arguments.printHelp();
			return 1;
		}

		if (arguments.exists("help")) {
			arguments.printHelp();
			return 0;
		}

		const std::string path = arguments.getString("path");
		const std::string output = arguments.exists("output") ? arguments.getString("output") : "*";

		Wallpapers wallpapers(path);

		if (!arguments.exists("never-random")) {
			wallpapers.shuffle();
		}

		if (arguments.exists("lock")) {
			Process swaylockProcess = makeSwaylockProcess(wallpapers.next());
			swaylockProcess.exec();
			swaylockProcess.wait();
			return 0;
		}

		Process swayBgProcess = makeSwaybgProcess(wallpapers.next(), output);
		swayBgProcess.exec();

		if (arguments.exists("interval")) {
			int interval = arguments.getInt("interval");
			while (true) {
				sleep(interval);
				Process newSwayBgProcess = makeSwaybgProcess(wallpapers.next(), output);
				newSwayBgProcess.exec();
				sleep(1);
				swayBgProcess.kill();
				swayBgProcess = newSwayBgProcess;
			}
		} else {
			char ch;
			while (ch != 'q') {
				ch = std::cin.get();
			}
		}

		swayBgProcess.kill();
		return 0;
	} catch (std::exception& e) {
		std::cerr << e.what() << std::endl;
		return 1;
	}
}
